@@ -8,7 +8,7 @@
  work, as well as functions which operate on them.
 """
 from types import FunctionType as Function
-from typing import Any, Dict, Hashable, List, Type, TypeVar, Union
+from typing import Any, Dict, Hashable, List, Optional, Type, TypeVar, Union
 
 DataClass = Any  # type hint for variables that should be data class instances
 
@@ -49,16 +49,16 @@ class DataClassMeta(type):
         all_defaults = {}
         all_slots = set()
         options = dict(mcs.DEFAULT_OPTIONS)
-        post_init = False  # whether a post-initialisation method was defined by the user
+        post_init: Optional[Function] = None  # a post-initialisation method if defined by the user
 
         dataclass_bases = [vars(b) for b in bases if hasattr(b, '__dataclass__')]
         for b in dataclass_bases + [dict_]:
             all_annotations.update(b.get('__annotations__', {}))
             all_defaults.update({f: v for f, v in b.get('__defaults__', dict_).items() if f in all_annotations})
-            all_slots.update(b.get('__slots__', set()))
+            all_slots.update(b.get('__slots__', []))
             options.update(b.get('__dataclass__', {}))
 
-            post_init = post_init or is_user_func(b.get('__init__')) or is_user_func(b.get('__post_init__'))
+            post_init = post_init or user_func(b.get('__init__')) or user_func(b.get('__post_init__'))
         options.update(kwargs)
 
         # fill out this class' dict and store defaults, annotations and decorator options for future subclasses
@@ -85,9 +85,9 @@ class DataClassMeta(type):
             del dict_['__slots__']
 
         if options['init'] and all_annotations:  # only generate __init__ if there are fields to set
-            if post_init and '__init__' in dict_:
-                dict_['__post_init__'] = dict_.pop('__init__')
-            dict_['__init__'] = generate_init(all_annotations, all_defaults, post_init,
+            if post_init:
+                dict_['__post_init__'] = dict_.pop('__init__', post_init)
+            dict_['__init__'] = generate_init(all_annotations, all_defaults, bool(post_init),
                                               options['kwargs'], options['frozen'])
 
         if options['repr']:
@@ -127,7 +127,7 @@ def generate_init(annotations: Dict, defaults: Dict, user_init: bool, gen_kwargs
     class. When the data class is initialised, arguments to this function are applied to the fields of the new instance.
     A user-defined __init__, if present, must be aliased to avoid conflicting."""
     arguments = [a for a in annotations if a not in defaults]
-    default_arguments = [f'{a}={a}' for a in annotations if a in defaults]
+    default_arguments = [f'{a}={a}' for a in defaults]
     args = ['*args'] if user_init else []
     kwargs = ['**kwargs'] if user_init or gen_kwargs else []
 
@@ -162,10 +162,10 @@ def eval_function(name: str, lines: List[str], annotations: Dict, locals_: Dict,
     return function
 
 
-def is_user_func(obj: Any) -> bool:
-    """Returns whether an object is a function explicitly defined by the user (i.e. not generated by
-    dataclassy and not a wrapper/descriptor)."""
-    return type(obj) is Function and not hasattr(obj, '__dataclass__')
+def user_func(obj: Any) -> Optional[Function]:
+    """Given an object, returns it if it is an object explicitly defined by the user (i.e. not generated by
+    dataclassy and not a wrapper/descriptor). Otherwise, returns None."""
+    return obj if type(obj) is Function and not hasattr(obj, '__dataclass__') else None
 
 
 # generic method implementations common to all data classes
